****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_extractmilinkage.sas  
*
* Created (mm/dd/yyyy): 10/20/2018
* Last modified: 11/12/2018 
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This program extracts MIL records based solely on Match Method and Singleton status. This macro will
*	perform both raw claim extraction to avoid claim duplication, and loop extraction
*
*  Program inputs:                                                                                   
*  	-dataset to extract records from
* 	-dataset containing the list of match method/singleton status values
*
*  Program outputs:                                                                                                                                       
*  	-dataset containing the extracted records
*             
* PARAMETERS:
*  	-datafile		= Name of the dataset to extract records from.
*  	-lookfile		= Name of the sas dataset containing the list of Match Method/Singleton codes to extract.  The name of the variable 
*           		  containing the codes must be called "code" and the one containing the codetype must be
*           		  called "codetype".
*  	-outfile		= Name of the output dataset containing the extracted records.
*
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    1.1	  11/12/18	  AP		Added distinct to proc sql to prevent a nodupkey sort in ms_cidanum
*
***************************************************************************************************;

%macro ms_extractmilinkage(datafile=, lookfile=, outfile=);

%put =====> MACRO CALLED: ms_extractmilinkage v1.1;

	%isdata(dataset=indata.&miltable.);	
    %if %eval(&nobs.=0) & "&datafile" = "indata.&miltable." %then %do;
        data &outfile.;
			length adate 4.;
	        set indata.&proctable.(obs=0 keep=Patid Adate)
	            &lookfile.(obs=0);
        	if _N_<1;
			csex="";
			linkedid = patid;
		   	enctype='';
			cbirth_date=.;
			cenr_start=.;
			birth_type=.;
			matchmethod='';
		    RxSup=.;
			RxAmt=.;
        run;
    %end;
    %else %do;

	    %ISDATA(dataset=&lookfile.);
		%if %eval(&NOBS.>=1) %then %do;
	        proc sql noprint ;
	        create table &outfile.(rename=date=adate where=(patid is not missing)) as
	        select distinct
				/*intiial extraction: assign patid to cohort of interest, retain linked ID*/
				%if "&datafile" = "indata.&miltable." %then %do;
				  case 
					when codes.codetype = 'M' then mpatid
					when codes.codetype = 'I' then cpatid
					else ''
					end as patid
				, case 
					when codes.codetype = 'M' then cpatid
					when codes.codetype = 'I' then mpatid
					else ''
					end as linkedid
				, claim.adate as date length =4
				, claim.sex as csex
				%end;
				%else %do;
				/*loop extraction: assign ADATE*/
				  claim.patid
				, claim.linkedid
			    , case when missing(claim.cbirth_date)=0 then cbirth_date
				   else adate
				   end as date length=4
				, claim.csex
				%end;
				, claim.enctype
				, claim.cbirth_date
				, claim.cenr_start
				, claim.birth_type
				, claim.matchmethod
			    , 1 as RxSup
				, 1 as RxAmt
				, codes.*
				, strip(put(claim.birth_type, 3.)) as charbirthtype
	        from &datafile. as claim,
	             &lookfile. as codes
	        where substr(codes.code,1,2) = claim.MatchMethod and
	              substr(codes.code,3,1) = calculated charbirthtype

				/*Remove records that do not meet GRACEDAYS and MIDAYSDIFF parameters*/
				/*Can equal '.' if non-linked and linked mothers selected*/
				%if "&datafile" ne "indata.&miltable." %then %do;
					and . <= abs(claim.cenr_start-claim.cbirth_date) <=&gracedays.
					and . <= abs(claim.cbirth_date-claim.adate)<=&midaysdiff.
				%end;
				  ;

	        quit;
	    %end;
	    %else %do;
	        data &outfile.;
	        set &datafile.(obs=0 %if "&datafile" = "indata.&miltable." %then %do; rename=cpatid=linkedid rename=mpatid=patid rename=sex=csex %end;) &lookfile.(obs=0);
	        run;
	    %end;

	%end;

    %put NOTE: ********END OF MACRO: ms_extractmilinkage v1.1********;

%mend ms_extractmilinkage;       
