****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_findgap.sas  
*
* Created (mm/dd/yyyy): 07/31/2014
* Last modified: 02/12/2019
* Version: 1.3
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This program will find any first date that is the first after a user 
*  	defined gap (or the very first). 
*
*  Program inputs:                                                                                   
*	-Dataset in which gaps are to be looked for 
* 
*  Program outputs:                                                                                                                                       
* 	-Dataset with the gap condition applied  
* 
* PARAMETERS:
*  	-InFile	= Name of the dataset in which gaps are to be found
*  	-OutFile= Name of the output dataset with the gap condition applied
*  	-sort	= List of variables used to sort InFile 
*  	-NoDup	= Duplicate records removal indicator used when sorting
*  	-gapby	= Group of variable that indicates how to look for gaps
*  	-gap	= Gap value to look for between records according to gapby
*  	-dupby	= Group of variable that indicates how to remove duplicates
*
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1		  01/19/18	  AP		Added deduplication of Adate based on max(expiredt)
*
*   1.2		  12/19/18	  AP		DEV-1037: added code to check for non-stockpiled dispensings in gap
*
*   1.3		  02/12/19	  AP		Fixed bug in code that prevented Type 4 MOI processing
*
***************************************************************************************************;

%macro ms_findgap(InFile=, OutFile=, sort=, NoDup= ,gapby=, gap=, dupby=);

	%put =====> MACRO CALLED: ms_findgap v1.3;

	data _null_;
	call symputx("lastwrd",scan("&gapby.",countw("&gapby.")));
	call symputx("lastwrddup",scan("&dupby.",countw("&dupby.")));
	if "&NoDup."="Y" then call symputx("nodupkey","nodupkey");
	else call symputx("nodupkey","");
	run;
	%put &lastwrd. &nodupkey.;

	proc sort data=&InFile. &nodupkey. out=&OutFile.;
	by &sort.;
	run;

	%if "&NoDup."="Y" %then %do;
	*remove duplicate adate, and keep longer duration (to ensure washout applied to longer duration);
	data &OutFile.;
		set &OutFile.;
		by &dupby.;
		if first.&lastwrddup.;
	run;
	%end;

	*Compute gaps;
	data &OutFile.;
	set &OutFile.;
	by &gapby.;
	DaysUntreated=Adate-lag(ExpireDt);
	gap=DaysUntreated > max(&gap.,0);
	if &gap.=. then gap=0;  *If washper=., then only first date is valid (formerly MIN in the MIN,SING, MULT sequence);
	if first.&lastwrd. then gap=1;  *first one always counts as being a potential index date;
	if gap or (&gap.=0);  * if washper =0 then all claims are deemed potential indedate wrt own self;
	drop gap;
	run;

	/*Need to reapply gap against all remaining claims (not just prior claim)*/
	/*if washper = 0, then all claims meet gap requirement*/
	%if %eval(&&WASHPER&paramsuffix. ne 0) %then %do;
		proc sql noprint undo_policy=none;
		    create table _findgap_incident as
		    select distinct index.* 
		    from &OutFile. as index, 
		         &OutFile. as Inc
		    where index.Patid = Inc.Patid and index.adate ne inc.adate and %MS_PeriodsOverlap(period1=index.Adate-min(index.&gap.,99999) index.Adate-1,
		                                       	period2=Inc.ADate Inc.ExpireDt)
		    order by index.Patid, index.adate;
		quit;

		data &OutFile.;
			merge &OutFile.(in=a) _findgap_incident(in=b);
			by patid adate;
			if a and not b;
		run;

		proc datasets nowarn noprint lib=work;
			delete _findgap_incident;
		quit;
	%end;

	%put NOTE: ********END OF MACRO: ms_findgap v1.3********;

%mend ms_findgap;
