****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_t1t2_createbaselinetable.sas  
*
* Created (mm/dd/yyyy): 12/27/2017
* Last modified: 07/18/2018
* Version: 1.2
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: The macro produces baseline table 1s
*                                        
*  Program inputs:                                                                                   
*  	- For level 1 requests: baseline tables in the format [RUNID]_baseline_[Look].sas7bdat
* 
*  Program outputs:                                                                                                                                       
* 
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1 	  03/27/18	 AP			Added defensive coding if no covariates in baseline table (QRP-514)
*
*   1.2 	  07/18/18	 AP			Changed &grp to &grps (DEV-1033)
*
*   1.3 	  12/28/18	 AP			Changed macro name from ms_createbaselinetable to ms_t1t2_createbaselinetable
*
***************************************************************************************************;

%macro ms_t1t2_createbaselinetable();

	%put =====> MACRO CALLED: ms_t1t2_createbaselinetable v2.0;

/************************************************************************************************************************************
*	1. Aggregate and produce DP level baseline tables for level 1 analysis							
************************************************************************************************************************************/

	/*loop through each look*/
	%do i = %eval(&look_start) %to %eval(&look_end);
		%let infile = baseline_&i.; 

			** Import and aggregate each of the baseline tables by group  ** ;
			%if %length(&T1GroupList.) > 0 %then %do;
				%do grps = 1 %to &NUM_T1Groups.;
					%let RUNID = %scan(&T1runids., &grps.);
					%let group = %scan(&T1GroupList,&grps.);
					/*to assign groups to the sort order in input file*/
					data _null_;
						order = put("&group.", $orderfmt.);
						call symputx("order", order);
					run;
				    %if %sysfunc(exist(msoc.&RUNID._&infile.))=0 %then %do;
					   	%put WARNING: &infile. does not exist. Baseline table will not be produced.;
						%let output_baselinetable = N;
						%goto exit; 
					%end;
					%else %do;													
						data temp_grp&order._&i.; 
							set msoc.&RUNID._&infile.; 
							where group = "&Group.";
						run;

						/*Transpose*/
						proc transpose data=temp_grp&order._&i. out=table1_grp&order._&i.; 
						id group;
						run;
					
						proc sort data =table1_grp&order._&i.; 
							by _name_; 
						run;

						*if no covars need to initialize _label_;
						data table1_grp&order._&i.; 
							set table1_grp&order._&i.;
								if _n_ = 1 then do;
									dsid = open("table1_grp&order._&i.");
									if varnum(dsid,"_label_") = 0 then do;
										format _label_ $70.;
										_label_ ='';
									end;
									rc= close(dsid);
								end;
							drop rc dsid;
						run;	
					%end;
				%end;
			%end;
			
			** Import and aggregate each of the baseline tables by combined groups (group1 and group2)  ** ;
			%if %length(&combinegroup1.) > 0 and %length(&combinegroup2.) > 0 %then %do;
				%do r = 1 %to &NUM_Crunids.;
					%put combiner resolves to ==> &r.;
						%do c = 1 %to 2;
							%put combiner  resolves to ==> &&combinerun&c..;
							%let RUNID&c. = %scan(&&combinerun&c.., &r.);
							%let GROUP&c. = %scan(&&combinegroup&c.., &r.);
						%end;
						%if %sysfunc(exist(msoc.&RUNID1._&infile.))=0 or %sysfunc(exist(msoc.&RUNID2._&infile.))=0 %then %do;
					    	%put WARNING: &infile. does not exist. Baseline table will not be produced.;
							%let output_baselinetable = N;
							%goto exit; 
			   			%end;
						%else %do;
							%do c = 1 %to 2;
								data combine_r&r._&c.;
									set msoc.&&RUNID&c.._&infile. ;								;
									where group in ("&&GROUP&c..");
								run;
							 %end;

							proc sql noprint;
								select combinedgroupname into: combinedgroupname
								from infolder.&groupfile.
								where group1 = "&group1." and group2 = "&group2.";
							quit;

							 data combine_r&r.;
							 	 length combinegrouplabel $61;
	  								set  combine_r&r._1 combine_r&r._2; 
								 	combinegrouplabel = "&combinedgroupname.";
									call symputx("group", combinegrouplabel);
									order = put(combinegrouplabel, $orderfmt.);
									call symputx("order", order);
							 run;

							**** Collapse data from Group 1 and Group 2 ***;

							/*for continuous variables, sum across combined groups*/
							%let dsid = %sysfunc(open(combine_r&r.));
							%let raceexist = ;
							%let hispanicexist = ;
							%let covarexist = ;
							%if %sysfunc(varnum(&dsid, race_0)) or %sysfunc(varnum(&dsid, race_1)) or %sysfunc(varnum(&dsid, race_2)) 
								or %sysfunc(varnum(&dsid, race_3)) or %sysfunc(varnum(&dsid, race_4)) or %sysfunc(varnum(&dsid, race_5))
								or %sysfunc(varnum(&dsid, race_6)) %then %do;
								%let raceexist = race:;
							%end;
							%if %sysfunc(varnum(&dsid, hispanic_y)) or %sysfunc(varnum(&dsid, hispanic_n)) or %sysfunc(varnum(&dsid, hispanic_u)) %then %do;
								%let hispanicexist = hispanic:;
							%end;
							%if %sysfunc(varnum(&dsid, covar1)) %then %do;
								%let covarexist = covar:;
							%end;
							%let rc=%sysfunc(close(&dsid));

							proc means data= combine_r&r. nway noprint;
								class combinegrouplabel ;
								var patient Age: Sex: Year: &covarexist. N_episodes &raceexist. &hispanicexist. ;
								output out=combine_r&r._summ (drop=_:) sum=;
							run;
								
							/*total episodes for each combinegroup*/
							data _null_;
								set combine_r&r._summ;
								call symputx("n_episodes", N_episodes);
							run;

							proc transpose data = combine_r&r. (keep =  mean_: std_:) out = _trans (drop = col:);
								var mean_: std_:;
							run;

							proc sql noprint;
								 select strip(_name_) into: meanvars separated by ' ' from _trans where index(upcase(_name_), 'MEAN') > 0;
								 select "wgt_"||strip(_name_) into: wgtmeanvars separated by ' ' from _trans where index(upcase(_name_), 'MEAN') > 0;
								 select strip(_name_) into: stdvars separated by ' ' from _trans where index(upcase(_name_), 'STD') > 0 ;
								 select "wgt_"||strip(_name_) into: wgtstdvars separated by ' ' from _trans where index(upcase(_name_), 'STD') > 0 ;
							quit;

							/*For mean variables, calculated the weighted mean for combined groups*/
							data weightedmean (keep = combinegrouplabel &wgtmeanvars.);	 
								set combine_r&r. (keep = combinegrouplabel N_episodes mean_:);
								array means{*} &meanvars.;
								array wgtmeans{*} &wgtmeanvars.;
									do i = 1 to dim(means);
										if ^missing(means{i}) then wgtmeans{i} = means{i}*n_episodes;
									end;
							run;
							proc means data = weightedmean nway noprint;
								 class combinegrouplabel;
								 var &wgtmeanvars.;
								output out = weightedmean (drop = _:) sum=;
							run;

							data weightedmean (keep = combinegrouplabel &meanvars.);
								 set weightedmean;
								 array means{*} &meanvars.;
								 array wgtmeans{*} &wgtmeanvars.;
								 do w = 1 to dim(means);
								 	if ^missing(wgtmeans{w}) then means{w} = wgtmeans{w}/&n_episodes.;
								 end;
							run;

							/*for stddev variables, calculated the pooled standard deviation for combined groups*/
							data weightedstddev;
								set combine_r&r. (keep = combinegrouplabel n_episodes &stdvars.);
									array stddev(*) &stdvars.;
									array wgtstddev(*) &wgtstdvars.;
									do i = 1 to dim(stddev);
										if ^missing(stddev{i}) then wgtstddev{i} = (stddev{i}**2)*(n_episodes-1);
									end;
							run;

							proc means data = weightedstddev nway noprint;
								class combinegrouplabel;
								var &wgtstdvars.;
								output out = weightedstddev (drop = _:) sum=;
							run;

							data weightedstddev (keep = combinegrouplabel &stdvars.);
								set weightedstddev;
									array stddev(*) &stdvars.;
									array wgtstddev(*) &wgtstdvars.;
									do i = 1 to dim(stddev);
										if ^missing(wgtstddev{i}) then stddev{i} = sqrt(wgtstddev{i}/(&n_episodes.-2));
									end;
							run;

							/*aggregated table for combined group*/
							data combine_r&r.;
								length group $61;
								merge combine_r&r._summ
									  weightedmean
									  weightedstddev;
								by combinegrouplabel;	
								group = combinegrouplabel;
							run;

							/*Transpose*/
							proc transpose data=combine_r&r. out= table1_grp&order._&i.; 
							id group;
							run;

							*if no covars need to initialize _label_;
							data table1_grp&order._&i.; 
								set table1_grp&order._&i.; 
									if _n_ = 1 then do;
										dsid = open("table1_grp&order._&i.");
										if varnum(dsid,"_label_") = 0 then do;
											format _label_ $70.;
											_label_ ='';
										end;
										rc= close(dsid);
									end;
								drop rc dsid;
							run;
						%end;
					%end;
				%end;

		/*Compute aggregated baseline tables*/
		%ms_compute_baselinetable();

		*Table of contents;
		/*Loop through groups and produce 1 table contents*/
		%do grps = 1 %to &Num_allgroups.;
			/*Assign group labels from input file*/
			%let grouplabel = %scan(%bquote(&allbaselinelabel.), &grps., %str(|)); 							

			%tableletter();
			data toctable1agg;
				length tables $10 caption $500;
				tables = "Table 1&tableletter.";
				caption = "Baseline table for &grouplabel.";
			run;
			proc append data = toctable1agg base = toctable1;	run;
		%end; /*loop through groups*/

		proc datasets nowarn nolist noprint lib=work; delete temp: tab_agg agg: dp: combine:; quit;

	%end; /*loop through looks*/

	%exit:

	%put =====> END MACRO: ms_t1t2_createbaselinetable v2.0;

%mend ms_t1t2_createbaselinetable;
